#include <mega128.h>
#include <delay.h>
#include <stdio.h>
#include <math.h>
#include <spi.h>
#include "def.h"
#include "LCD_3320.h"
#include "menu.h" 
#include "displ.h"
#include "coder.h" 

//    EEPROM'
eeprom MODEL_SET EEP_models[MAX_MODELS];
eeprom unsigned char EEP_start_model;
eeprom unsigned char EEP_f_sound_en;
eeprom unsigned char EEP_BatCntr[3];
eeprom int EEPR_ADmid[5], EEPR_ADmax[5], EEPR_ADmin[5];  
eeprom unsigned char EEP_FirstON, EEP_Calibration;
eeprom signed int EPP_CalibrTemperature[2];
//------------------------------------------------------
//extern unsigned char WidthS, HeightS;

signed int out_control[MAX_CONTRS];
signed int ADmid[5];
signed int ADmin[5];
signed int ADmax[5];
signed int out_ch[MAX_CHANS];
//signed int VOut[5];
MODEL_SET cur_model;
unsigned char cur_model_ind;
unsigned char cur_mode_ind;
bit f_TCut;
unsigned int cur_Batt;
unsigned int cur_Power;
unsigned int cur_Temperature;
signed int dir_Temperature;
unsigned char wd_Temperature;
unsigned int div_trimmer;
unsigned char f_l;
unsigned char g_MainMenuMode;
extern bit f_timer_en;
unsigned int gRPMtmp=0;
unsigned int gRPM[SIZE_RPM_BUFF];
unsigned char gRPMn;
signed int g_pos_d[3];
unsigned char g_keys; 
#define F_SW1 1
#define F_SW2_1 2
#define F_SW2_2 4
#define F_SW3 8
#define F_DUAL1 0x10
#define F_DUAL2 0x20
#define F_DUAL3 0x40
#define F_TCUT 0x80






//------------------------------------------------------
//------------------------------------------------------
#define ADC_VREF_TYPE 0x00
unsigned int read_adc(unsigned char adc_input)
{
ADMUX=adc_input|ADC_VREF_TYPE;
// Start the AD conversion
ADCSRA|=0x40;
// Wait for the AD conversion to complete
while ((ADCSRA & 0x10)==0);
ADCSRA|=0x10;
return ADCW;
}
//------------------------------------------------------
//------------------------------------------------------
// (PD0-tachometr)
interrupt [EXT_INT0] void ext_int0_isr(void)
{
gRPMtmp++;
if(gRPMtmp==1024) EIMSK&=0xfe;
}
//------------------------------------------------------
void TimerTachometer(void)
{ // 50Hz
static unsigned char div;
div++;
if(div>=25)
  { // ~2Hz
  div=0;
  gRPM[gRPMn]=gRPMtmp;
  if(++gRPMn>=SIZE_RPM_BUFF) gRPMn=0;
  gRPMtmp=0;
  }
}
//------------------------------------------------------
//------------------------------------------------------
__flash const unsigned int notes[]= {//Note A 1-okt;
440, 466, 494, 523,    554,  588,  612,  660,    698,  740,  784,  830, 
880, 932, 988, 1064,   1108, 1176, 1224, 1320,   1396, 1480, 1568, 1660};
BEEP_TRACK_DEF flash *beep_track;
unsigned int beep_length;
unsigned char beep_cnt;
unsigned char beep_cntn;
unsigned char beep_rep;
unsigned int beep_freq;
bit f_beep_fase;
bit f_beep_timer;
bit f_beep_EEP;
bit f_beep_key;
bit f_beep_trimmer;

extern flash const BEEP_TRACK_DEF hello_sound[];
//------------------------------------------------------
void beep(BEEP_TRACK_DEF flash *note)
{
unsigned long ln, freq;

freq=notes[note->note]; freq*=2L;
ln=1500000L/freq; ln=65536L-ln;
beep_freq=ln;
TCNT3H=beep_freq>>8;
TCNT3L=beep_freq & 0xff;
ln=note->ms*freq; ln/=1000L;
beep_length=ln;
}
//------------------------------------------------------
// Timer 3 overflow interrupt service routine
//------------------------------------------------------
interrupt [TIM3_OVF] void timer3_ovf_isr(void)
{
bit f_con;

TCNT3H=beep_freq>>8;
TCNT3L=beep_freq & 0xff;
if(--beep_length==0)
  {
  f_con=0;
  if(++beep_cntn<beep_cnt)  f_con=1;
  else if(beep_rep) { f_beep_fase=0; beep_rep--; beep_cntn=0; f_con=1; }
  if(f_con==0) { BUZ0; ETIMSK&=~0x4; return; }
  else beep((beep_track+beep_cntn));
  }
f_beep_fase=!f_beep_fase;
if(f_beep_fase) BUZ1; else BUZ0;
}
//------------------------------------------------------
void beep_start(BEEP_TRACK_DEF flash *notes, unsigned char cnt, unsigned char rep)
{
beep_track=notes;
beep_cnt=cnt;
beep_cntn=0;
beep_rep=rep;
beep(notes);
ETIMSK|=0x4;
}
//-------------------------------------------------------------
//-------------------------------------------------------------
//-------------------------------------------------------------
unsigned int CHcalc (char ch_ind)
{
signed long ln, ret;
unsigned char i;
signed char *chans;

chans=cur_model.modes[cur_mode_ind].chanels[ch_ind];
ret=2250; // 1,5 ms
for (i=0; i<MAX_CONTRS; i++) 
  {
  if (chans[i]!=0)
    { ln=out_control[i]; ln*=chans[i]; ln/=100; ret+=ln; }
  }  
if (ret>3150) ret=3150; //   2,1  
if (ret<1350) ret=1350; //   0,9 
return (unsigned int)ret;
}
//-------------------------------------------------------------
signed int interpol(signed int val, signed char* nodes, )
{
unsigned char n;
signed int n1, n2, vmin;
signed long ln;

ln=val; ln*=6; n=ln/1500;
n1=*(nodes+n); n1=(n1*75)/10;
n2=*(nodes+n+1); n2=(n2*75)/10;
vmin=n; vmin*=250;
ln=(val-vmin); ln*=(n2-n1); ln/=250;
return ln + n1;
}
//-------------------------------------------------------------
signed int rate_calc(signed int val, unsigned char min, unsigned char max)
{
unsigned char rate;
signed long v;

if(val<0) rate=min;
else rate=max;
v=val; v*=rate; v/=100;
return v;  
}
//-------------------------------------------------------------
unsigned char ReadKeys(void)
{
unsigned char f;

f=0;
if(!SW1) f|=F_SW1;
if(!SW2_1) f|=F_SW2_1;
if(!SW2_2) f|=F_SW2_2;
if(!SW3) f|=F_SW3;
if(!DUAL_ail) f|=F_DUAL1;
if(!DUAL_elev) f|=F_DUAL2;
if(!DUAL_rud) f|=F_DUAL3;
if(!Tcut_KEY) f|=F_TCUT;
return f;
}
//-------------------------------------------------------------
void Calc (void)
{ // 50Hz (20ms)
static unsigned char div_trim;
signed char *trim;
signed int input[10];
unsigned int i, n;
signed long vl;
const unsigned char acntr[]={ ch_Ailerons, ch_Elevator, ch_Throttle, ch_Rudder, ch_Rpop_contr };
signed int vi;
unsigned char mn, mx;
DEF_MODE *mode;
unsigned char f_dual[3];
unsigned char keys;
bit f;

//   
if(!MODE_KEY2)           i=0;
else if(!MODE_KEY1)      i=2;
else                     i=1;
if(cur_mode_ind!=i) { BEEP; save_trimmer(); cur_mode_ind=i; }

// 
trim=cur_model.modes[cur_mode_ind].trimmers; 
if (div_trim)
  {
  f=0;
  if(!trim_0up && trim[0]<80) { trim[0]++; f=1;  }
  if(!trim_0down && trim[0]>-80) { trim[0]--;  f=1; }
  if(!trim_1up && trim[1]<80) { trim[1]++; f=1; }
  if(!trim_1down && trim[1]>-80) { trim[1]--; f=1; }
  if(!trim_2up && trim[2]<80) { trim[2]++;   f=1; }
  if(!trim_2down && trim[2]>-80) { trim[2]--; f=1; }
  if(!trim_3up && trim[3]<80) { trim[3]++; f=1; }
  if(!trim_3down && trim[3]>-80) { trim[3]--; f=1; }
  if(f) { f_beep_trimmer=1; div_trimmer=1500; }
  }
div_trim=!div_trim;

mode=&cur_model.modes[cur_mode_ind];
//  
for(i=0; i<5; i++)
  {
  n=i; if(i==4) n=7;
  vl=read_adc(acntr[i]);
  if(vl<ADmid[i])
    {
    if(ADmin[i]>vl) ADmin[i]=vl;
    vl-=ADmin[i]; vl*=750; input[n]=vl/(ADmid[i]-ADmin[i]);
    }
  else
    {
    if(ADmax[i]<vl) ADmax[i]=vl;
    vl-=ADmid[i]; vl*=750; input[n]=750+(vl/(ADmax[i]-ADmid[i]));
    }
  }
  
keys=ReadKeys(); if(keys!=g_keys) { BEEP; g_keys=keys; }
if(keys & F_SW1) input[4]=0; else input[4]=1500;
if(keys & F_SW2_1) input[5]=0; 
else if(keys & F_SW2_2) input[5]=1500;
else input[5]=750;
if(keys & F_SW3) input[6]=0; else input[6]=1500;
for(i=0; i<3; i++)
  {
  vi=mode->contr_d[i].delay;
  if(vi==0) g_pos_d[i]=input[4+i];
  else if(g_pos_d[i]!=input[4+i])
    {
    vl=g_pos_d[i]; vl*=vi; vl/=(300/2); vl++; vl/=2;
    if(g_pos_d[i]<input[4+i])
      {
      vl++; vl*=(300*2); vl/=vi; vl++; vl/=2;
      if(vl>input[4+i]) vl=input[4+i];
      } 
    else
      {
      vl--; vl*=(300*2); vl/=vi; vl++; vl/=2;
      if(vl<input[4+i]) vl=input[4+i];
      }
    g_pos_d[i]=vl; input[4+i]=vl;
    }
  input[4+i]-=750;     
  }
input[8]=input[mode->contr_v[0].from];
input[9]=input[mode->contr_v[1].from];
//   
if (keys & F_TCUT) f_TCut=1; else f_TCut=0;
//  
f_dual[0]=(keys & F_DUAL1);
f_dual[1]=(keys & F_DUAL2);
f_dual[2]=(keys & F_DUAL3);
//  
for(i=0; i<3; i++)
  {
  n=i; if(i==2) n++;
  vi=interpol(input[n], mode->contr_ab[i].cur_nodes);
  if(f_dual[i]) { mn=mode->contr_ab[i].minDRates; mx=mode->contr_ab[i].maxDRates; }
  else { mn=mode->contr_ab[i].minRates; mx=mode->contr_ab[i].maxRates; }
  out_control[n]=rate_calc(vi, mn, mx);
  vi=mode->trimmers[n]; vi*=5; out_control[n]+=vi;
  }
if(f_TCut) { vi=mode->contr_tr.tcut; vi*=-75; out_control[2]=vi/10; }
else 
  {
  vi=interpol(input[2], mode->contr_tr.cur_nodes);
  out_control[2]=rate_calc(vi, mode->contr_tr.minRates, mode->contr_tr.maxRates);
  vi=mode->trimmers[2]; vi*=5; out_control[2]+=vi;
  }
if(cur_model.timer_mode==2) { if(!f_TCut && input[2]>150) f_timer_en=1; else f_timer_en=0; }
  
for(i=0; i<3; i++)
  out_control[4+i]=rate_calc(input[4+i], mode->contr_d[i].minRates, mode->contr_d[i].maxRates);

vi=interpol(input[7], mode->contr_ap.cur_nodes);
out_control[7]=rate_calc(vi, mode->contr_ap.minRates, mode->contr_ap.maxRates);

for(i=0; i<2; i++)
  {
  vi=interpol(input[8+i], mode->contr_v[i].cur_nodes);
  out_control[8+i]=rate_calc(vi, mode->contr_v[i].minRates, mode->contr_v[i].maxRates);
  }

for(i=0; i<MAX_CONTRS; i++) out_control[i]*=cur_model.contrs_reverse[i];
} 
//------------------------------------------------------
interrupt [TIM1_COMPA] void timer1_compa_isr(void)
{ 
static unsigned char nb;
static unsigned int sum;
unsigned int out;

if (nb<cur_model.num_ch)
  {       
  out=CHcalc(nb);
  ICR1H=out>>8;        //  
  ICR1L=out&0xff;
  out_ch[nb]=out;
  sum+=out; nb++;
  if(nb==1) TimerTachometer();
  }
else
  {
  out=30000-sum; //1500,000/30000= 50Hz
  ICR1H=out>>8; 
  ICR1L=out&0xff;     //  
  nb=0; sum=0;
#asm("sei")
  f_l++;  
  Calc();
  TimerOn();
  f_l++;
  }
}
//------------------------------------------------------
//------------------------------------------------------
//------------------------------------------------------
void main(void)
{
unsigned char i;
// Input/Output Ports initialization
// Port A initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State7=P State6=P State5=P State4=P State3=P State2=P State1=P State0=P 
PORTA=0xFF;
DDRA=0x00;
// Port B initialization
// Func7=Out Func6=In Func5=Out Func4=In Func3=In Func2=Out Func1=Out Func0=In 
// State7=0 State6=T State5=0 State4=T State3=T State2=0 State1=0 State0=T 
PORTB=0x01;
DDRB=0xA6;
// Port C initialization
// Func7=Out Func6=Out Func5=Out Func4=Out Func3=In Func2=In Func1=In Func0=In 
// State7=0 State6=0 State5=0 State4=0 State3=T State2=T State1=T State0=T 
PORTC=0x0F;
DDRC=0xF0;
// Port D initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State7=P State6=P State5=P State4=P State3=T State2=T State1=T State0=T 
PORTD=0xF0;
DDRD=0x00;
// Port E initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State7=P State6=P State5=P State4=P State3=P State2=P State1=T State0=T 
PORTE=0xFC;
DDRE=0x00;
// Port F initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State7=T State6=T State5=T State4=T State3=T State2=T State1=T State0=T 
PORTF=0x00;
DDRF=0x00;
// Port G initialization
// Func4=Out Func3=Out Func2=Out Func1=In Func0=In 
// State4=0 State3=0 State2=0 State1=T State0=T 
PORTG=0x00;
DDRG=0x1C;
// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: Timer 0 Stopped
// Mode: Normal top=FFh
// OC0 output: Disconnected
ASSR=0x00;
TCCR0=0x00;
TCNT0=0x00;
OCR0=0x00;
// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: 1500,000 kHz
// Mode: Fast PWM top=ICR1
// OC1A output: Non-Inv.
// OC1B output: Discon.
// OC1C output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
// Timer 1 Overflow Interrupt: Off
// Input Capture Interrupt: Off
// Compare A Match Interrupt: On
// Compare B Match Interrupt: Off
// Compare C Match Interrupt: Off
TCCR1A=0x82;
TCCR1B=0x1A;
TCNT1H=0x00;
TCNT1L=0x00;
ICR1H=0x17;
ICR1L=0x70;
OCR1AH=0x02;
OCR1AL=0x68;
OCR1BH=0x00;
OCR1BL=0x00;
OCR1CH=0x00;
OCR1CL=0x00;
// Timer/Counter 2 initialization
// Clock source: System Clock
// Clock value: Timer 2 Stopped
// Mode: Normal top=FFh
// OC2 output: Disconnected
TCCR2=0x00;
TCNT2=0x00;
OCR2=0x00;
// Timer/Counter 3 initialization
// Clock source: System Clock
// Clock value: 1500,000 kHz
// Mode: Normal top=FFFFh
// Noise Canceler: Off
// Input Capture on Falling Edge
// OC3A output: Discon.
// OC3B output: Discon.
// OC3C output: Discon.
// Timer 3 Overflow Interrupt: On
// Input Capture Interrupt: Off
// Compare A Match Interrupt: Off
// Compare B Match Interrupt: Off
// Compare C Match Interrupt: Off
TCCR3A=0x00;
TCCR3B=0x02;
TCNT3H=0x00;
TCNT3L=0x10;
ICR3H=0x00;
ICR3L=0x00;
OCR3AH=0x00;
OCR3AL=0x00;
OCR3BH=0x00;
OCR3BL=0x00;
OCR3CH=0x00;
OCR3CL=0x00;
// External Interrupt(s) initialization
// INT0: On
// INT0 Mode: Falling Edge
// INT1: Off
// INT2: Off
// INT3: Off
// INT4: Off
// INT5: Off
// INT6: Off
// INT7: Off
EICRA=0x02;
EICRB=0x00;
EIMSK=0x0;//0x01;
EIFR=0x01;
// Timer(s)/Counter(s) Interrupt(s) initialization
TIMSK=0x01;
ETIMSK=0x00;
// Analog Comparator initialization
// Analog Comparator: Off
// Analog Comparator Input Capture by Timer/Counter 1: Off
ACSR=0x80;
SFIOR=0x00;
// ADC initialization
// ADC Clock frequency: 187,500 kHz
// ADC Voltage Reference: AREF pin
ADMUX=ADC_VREF_TYPE;
ADCSRA=0x86;
/*
// SPI initialization
// SPI Type: Master
// SPI Clock Rate: 3000,000 kHz
// SPI Clock Phase: Cycle Half
// SPI Clock Polarity: Low
// SPI Data Order: MSB First
SPCR=0x50;
SPSR=0x00;
 */
// SPI initialization
// SPI Type: Master
// SPI Clock Rate: 2*3000,000 kHz
// SPI Clock Phase: Cycle Half
// SPI Clock Polarity: Low
// SPI Data Order: MSB First
SPCR=0x50;
SPSR=0x01;

#asm("sei")
LCD_init();
LCD_clear();    //*******************************************
LCD_init();
if (EEP_FirstON!=0xa5 || (!KEY_LEFT && !KEY_RIGHT)) { TX_reset(); EEP_FirstON=0xa5; }
if (EEP_Calibration!=0xa5 || !KEY_LEFT)
  {
  while(!KEY_LEFT || !KEY_RIGHT || !KEY_UP || !KEY_DOWN) delay_ms(100);
  calibr();
  while(!KEY_LEFT || !KEY_RIGHT || !KEY_UP || !KEY_DOWN) delay_ms(100);
  calibr_thermometer();
  }
while(!KEY_LEFT || !KEY_RIGHT || !KEY_UP || !KEY_DOWN) delay_ms(100);
g_MainMenuMode=0;
for(i=0; i<3; i++) g_pos_d[i]=0;
TX_init();
g_keys=ReadKeys();
if(!MODE_KEY2) cur_mode_ind=0;
else if(!MODE_KEY1) cur_mode_ind=2;
else cur_mode_ind=1;
cur_Batt=read_adc(ch_Batt);
cur_Power=read_adc(ch_Power);
cur_Temperature=read_adc(ch_Temperature);

Calc();  // c;
beep_start(hello_sound, 4, 0);
displ_menu();
delay_ms(500); 
TIMSK|=0x10; //   

while (1)
  {
//  keys_menu();
  displ_menu();
  }
}
//-------------------------------------------------------------
void calibr (void)
{
unsigned char i, x, y;
unsigned long v;
signed int AD[5], ADM;
signed int v11, v12, v13, v21, v22, v23;  
char buff[20]; //********************************************
   
beep_start(calibr_sound, 3, 0);
delay_ms(10);
AD[0]=read_adc(ch_Ailerons);
AD[1]=read_adc(ch_Elevator);
AD[2]=read_adc(ch_Throttle);
AD[3]=read_adc(ch_Rudder);
AD[4]=read_adc(ch_Rpop_contr);

for (i=0; i<5; i++) { ADmin[i]=AD[i]; ADmax[i]=AD[i]; }
f_sound_en=1;
  
while(KEY_RIGHT && KEY_LEFT)
  {
  LCD_clear();
  displ_head("Calibration");
  AD[0]=read_adc(ch_Ailerons);
  AD[1]=read_adc(ch_Elevator);
  AD[2]=read_adc(ch_Throttle);
  AD[3]=read_adc(ch_Rudder);
  AD[4]=read_adc(ch_Rpop_contr);
  
  for (i=0; i<5; i++)
    {
    if (ADmin[i]>AD[i]) ADmin[i]=AD[i];
    if (ADmax[i]<AD[i]) ADmax[i]=AD[i];
    }
    
//  LCD_rectangle(46,10, 76, 40, 1); // .  
//  LCD_rectangle(45, 9, 77, 41, 1);
  if(ADmax[0]-ADmin[0]<512) { LCD_dashline(46, 10, 46, 40); LCD_dashline(76, 10, 76, 40); }
  else { LCD_line(46, 10, 46, 40, 1); LCD_line(76, 10, 76, 40, 1); }
  if(ADmax[1]-ADmin[1]<512) { LCD_dashline(46, 10, 76, 10); LCD_dashline(46, 40, 76, 40); }
  else { LCD_line(46, 10, 76, 10, 1); LCD_line(46, 40, 76, 40, 1); }  
  v=AD[0]-ADmin[0]; v*=28; v/=(ADmax[0]-ADmin[0]); x=47+v;
  v=AD[1]-ADmin[1]; v*=28; v/=(ADmax[1]-ADmin[1]); y=39-v;
  LCD_pixel(x, y-2,1);        
  LCD_line(x-1, y-1, x+1, y-1, 1); 
  LCD_line(x-2, y, x+2, y,1); 
  LCD_line(x-1, y+1, x+1, y+1, 1); 
  LCD_pixel(x, y+2, 1);
  if(ADmax[0]-ADmin[0]>=512 && ADmax[1]-ADmin[1]>=512)
    { 
    v11=(ADmax[0]-ADmin[0])/2; v12=v11/10; /* +- 10% */ v13=AD[0]-ADmin[0];
    v21=(ADmax[1]-ADmin[1])/2; v22=v21/10; /* +- 10% */ v23=AD[1]-ADmin[1];
    if( v13>(v11-v12) && v13<(v11+v12) && v23>(v21-v22) && v23<(v21+v22) )
      LCD_rectangle(x-3, y-3, x+3, y+3, 1);
    }
    
//  LCD_rectangle(7,10, 37, 40, 1); //  
//  LCD_rectangle(6,9, 38, 41, 1);
  if(ADmax[3]-ADmin[3]<512) { LCD_dashline(7, 10, 7, 40); LCD_dashline(37, 10, 37, 40); }
  else { LCD_line(7, 10, 7, 40, 1); LCD_line(37, 10, 37, 40, 1); }
  if(ADmax[2]-ADmin[2]<512) { LCD_dashline(7, 10, 37, 10); LCD_dashline(7, 40, 37, 40); }
  else { LCD_line(7, 10, 37, 10, 1); LCD_line(7, 40, 37, 40, 1); }    
  v=AD[3]-ADmin[3]; v*=28; v/=(ADmax[3]-ADmin[3]); x=8+v;
  v=AD[2]-ADmin[2]; v*=28; v/=(ADmax[2]-ADmin[2]); y=39-v;
  LCD_pixel(x, y-2,1);        
  LCD_line(x-1, y-1, x+1, y-1,1); 
  LCD_line(x-2, y, x+2, y,1); 
  LCD_line(x-1, y+1, x+1, y+1,1); 
  LCD_pixel(x, y+2,1);
  if(ADmax[3]-ADmin[3]>=512 && ADmax[2]-ADmin[2]>=512)
    { 
    v11=(ADmax[3]-ADmin[3])/2; v12=v11/10; /* +- 10% */ v13=AD[3]-ADmin[3];
    v21=(ADmax[2]-ADmin[2])/2; v22=v21/10; /* +- 10% */ v23=AD[2]-ADmin[2];
    if( v13>(v11-v12) && v13<(v11+v12) && v23>(v21-v22) && v23<(v21+v22) )
      LCD_rectangle(x-3, y-3, x+3, y+3, 1);
    }
    
//  LCD_rectangle(7, 43, 76, 47, 1);
  LCD_line(7, 43, 76, 43, 1); LCD_line(7, 47, 76, 47, 1);
  if(ADmax[4]-ADmin[4]<512) { LCD_dashline(7, 43, 7, 47); LCD_dashline(76, 43, 76, 47); }
  else { LCD_line(7, 43, 7, 47, 1); LCD_line(76, 43, 76, 47, 1); }
  v=AD[4]-ADmin[4]; v*=67; v/=(ADmax[4]-ADmin[4]); x=8+v;
  LCD_fillrect(x-1, 43, x+1, 47, 1);
  if(ADmax[4]-ADmin[4]>=512)
    {
    v11=(ADmax[4]-ADmin[4])/2; v12=v11/10; /* +- 10% */ v13=AD[4]-ADmin[4];
    if( v13>(v11-v12) && v13<(v11+v12))
      LCD_rectangle(x-3, 43, x+3, 47, 1);
    }
    
  /*  
  AD=read_adc(ch_Temperature);
  sprintf(buff, "%u", AD);
  LCD_setxy(5,24); LCD_wrs(buff, 1);
  LCD_refresh();
  */ 
  //************  
  ADM=AD[0];
  sprintf(buff, "%u", ADM);
  LCD_setxy(5,49); LCD_wrs(buff, 1);
  ADM=AD[1];
  sprintf(buff, "%u", ADM);
  LCD_setxy(30,49); LCD_wrs(buff, 1);
  ADM=AD[2];
  sprintf(buff, "%u", ADM);
  LCD_setxy(5,57); LCD_wrs(buff, 1);
  ADM=AD[3];
  sprintf(buff, "%u", ADM);
  LCD_setxy(30,57); LCD_wrs(buff, 1);
  ADM=AD[4];
  sprintf(buff, "%u", ADM);
  LCD_setxy(60,53); LCD_wrs(buff, 1);    
  LCD_refresh(); 
  //************  
        
  }
if(!KEY_LEFT) return; 
ADmid[0]=read_adc(ch_Ailerons);
ADmid[1]=read_adc(ch_Elevator);
ADmid[2]=read_adc(ch_Throttle);
ADmid[3]=read_adc(ch_Rudder);
ADmid[4]=read_adc(ch_Rpop_contr);
for (i=0; i<5; i++)
  {
  EEPR_ADmin[i]=ADmin[i];
  EEPR_ADmid[i]=ADmid[i];
  EEPR_ADmax[i]=ADmax[i];
  }


for (i=0, x=0; i<5; i++)
  {
  //if(ADmax[i]-ADmin[i]<512) { x=1; break; }
  if(ADmax[i]-ADmin[i]<256) { x=1; break; }
  
  v11=(ADmax[i]-ADmin[i])/2; 
  //v12=v11/25; // +- 25% 
  v12=v11/16; // +- 10%     //  ******************************************
  v13=ADmid[i]-ADmin[i];
  if( v13<(v11-v12) || v13>(v11+v12) ) { x=1; break; }
  }

  
if(x)
  {
  LCD_clear();
  LCD_setxy(8,11);
  LCD_wrsf("Calibration", 1);
  LCD_setxy(8,20);
  LCD_wrsf("not correct!", 1);
  LCD_refresh();
  beep_start(error_sound, 2, 0);
  delay_ms(2000);
  EEP_Calibration=0;
  return;
  }  
EEP_Calibration=0xa5;  
}
//-------------------------------------------------------------
void calibr_thermometer (void)
{
signed int t;
unsigned int AD;
char buff[20];
bit f;
   
t=25; f=0;
while(KEY_RIGHT && KEY_LEFT)
  {
  if(!KEY_UP || !KEY_DOWN)
    {
    beep_start(key_sound, 1, 0);
    if(!KEY_UP && t<100) t++;
    if(!KEY_DOWN && t>-25) t--;
    f=1;
    } 
  LCD_clear();
  displ_head("Thermometer");
  sprintf(buff, "Set curr %c: %d", 131, t);
  LCD_setxy(5,11); LCD_wrs(buff, 1);
  AD=read_adc(ch_Temperature);
  sprintf(buff, "%u", AD);
  LCD_setxy(5,24); LCD_wrs(buff, 1);
  LCD_refresh();
  if(f) { f=0; while(!KEY_UP || !KEY_DOWN) delay_ms(100); }
  }
if(!KEY_LEFT) return;
EPP_CalibrTemperature[0]=50+t;  
EPP_CalibrTemperature[1]=AD;
}       
//-------------------------------------------------------------
void TX_reset (void)
{
unsigned char i, x;
unsigned int v;

displ_head("Initial");
LCD_rectangle(8, 29, 75, 38, 1);
LCD_refresh();
for(i=0, x=8; i<MAX_MODELS; i++)
  {
  MODEL_reset(i);
  v=(75-8)*(unsigned int)i; v/=(MAX_MODELS-1); x=(unsigned char)(8+v);
  LCD_fillrect(8, 29, x, 38, 1);
  LCD_refresh();
  }
EEP_start_model=0;
EEP_f_sound_en=1;
EEP_BatCntr[0]=98; EEP_BatCntr[1]=92; EEP_BatCntr[2]=110;
}
//-------------------------------------------------------------
